"""Base components."""

from .body import Body
from .document import DocumentHead, Html, Main, Script, ColorModeScript
from .head import Head
from .link import Link
from .meta import Description, Image, Title
